#ifndef LSTMNETWORK__H
#define LSTMNETWORK__H

#include "NeuralNetwork.hpp"
#include "ContextLSTM.hpp"
#include "RawInputLSTM.hpp"
#include "SplitTransLSTM.hpp"
#include "FocusedColumnLSTM.hpp"

class LSTMNetworkImpl : public NeuralNetworkImpl
{
  private :

  torch::nn::Embedding wordEmbeddings{nullptr};
  torch::nn::Dropout embeddingsDropout{nullptr};
  torch::nn::Dropout lstmDropout{nullptr};
  torch::nn::Dropout hiddenDropout{nullptr};
  torch::nn::Linear linear1{nullptr};
  torch::nn::Linear linear2{nullptr};

  ContextLSTM contextLSTM{nullptr};
  RawInputLSTM rawInputLSTM{nullptr};
  SplitTransLSTM splitTransLSTM{nullptr};
  std::vector<FocusedColumnLSTM> focusedLstms;

  bool hasRawInputLSTM{false};

  public :

  LSTMNetworkImpl(int nbOutputs, int unknownValueThreshold, std::vector<int> bufferContext, std::vector<int> stackContext, std::vector<std::string> columns, std::vector<int> focusedBufferIndexes, std::vector<int> focusedStackIndexes, std::vector<std::string> focusedColumns, std::vector<int> maxNbElements, int leftWindowRawInput, int rightWindowRawInput, int embeddingsSize, int hiddenSize, int contextLSTMSize, int focusedLSTMSize, int rawInputLSTMSize, int splitTransLSTMSize, int numLayers, bool bilstm, float lstmDropout);
  torch::Tensor forward(torch::Tensor input) override;
  std::vector<std::vector<long>> extractContext(Config & config, Dict & dict) const override;
};

#endif
